//! Expression module

#![allow(warnings)]

mod opcode;
mod parser;
mod program;
mod runtime;
mod value;

type Register = usize;
type Addr = usize;

#[derive(thiserror::Error, Debug)]
pub enum Error {
    #[error("Program is out of bound")]
    OutOfBoundaries,

    #[error("Register {0} is out of boundaries")]
    RegisterOutOfBoundaries(Register),

    #[error("Empty registers")]
    EmptyRegisters,

    #[error("Value is not a number")]
    NotANumber,

    #[error("Math overflow")]
    Overflow,
}

pub use self::program::Program;
