use super::{value::Value, Addr, Register};

#[derive(Clone, Debug, PartialEq)]
pub enum OpCode {
    /// LOAD <destination> <value>
    LOAD(Register, Value),
    /// Copy the value from the source register to the destination register
    /// CPY <destination> <source>
    CPY(Register, Register),
    /// Move the value from the source register to the destination register
    /// MOV <destination> <source>
    MOV(Register, Register),
    /// Compare two registers and store the result in the destination register
    /// CMP <destination> <source1> <source2>
    CMP(Register, Register, Register),
    /// ADD <destination> <source1> <source2>
    ADD(Register, Register, Register),
    /// ADD <destination> <source1> <source2>
    AND(Register, Register, Register),
    OR(Register, Register, Register),
    XOR(Register, Register, Register),
    NOT(Register, Register),
    /// JMP <address>
    JMP(Addr),
    /// JEQ <register> <address>
    ///
    /// Jumps if the value in the register is true
    JEQ(Register, Addr),
    /// JNE <register> <address>
    ///
    /// Jumps if the value in the register is false
    JNE(Register, Addr),
    HLT,
}
