use super::{opcode::OpCode, parser::parse_query, runtime::execute, value::Value, Addr, Error};
use crate::Transaction;

#[derive(Debug, Clone)]
pub struct Program {
    /// The list of opcodes that make up the program
    opcodes: Vec<OpCode>,
    /// If the program has some boilerplate that can be skipped. This is non-zero when the initial
    /// program has been executed at compile time, and the register has been populated to
    /// `initial_register`. Everytime a new Runtime is created, instead of executing the program,
    /// it can safely start-off a different Address, and clone the initial_register.
    start_at: Addr,
    /// The state of the register
    initial_register: Vec<Value>,
}

impl Program {
    pub fn new(code: &str) -> Result<Self, Error> {
        let ast = parse_query(code);
        panic!("{:#?}", ast);
        todo!()
    }

    /// Creates a program tailored for a transaction
    pub fn execute(&self, transaction: Option<&Transaction>) -> Result<Value, Error> {
        execute(
            transaction,
            &self.opcodes,
            self.initial_register.clone(),
            self.start_at,
        )
    }
}

#[cfg(test)]
mod test {
    use super::Program;
    use crate::filter_expr::{opcode::OpCode, value::Value};

    #[test]
    fn parse() {
        let x = Program::new(
            r#"
            WHERE
                $foo = 3
                AND $bar
                AND $foo = 1 + 2 + 3 +  ((1+ 2-3*4-5) * $bar.tx.lol)
            LIMIT 10
            ORDER BY $foo DESC
        "#,
        );
    }

    #[test]
    fn simple_program() {
        let program = Program {
            opcodes: vec![
                OpCode::LOAD(1, 12.into()),
                OpCode::LOAD(2, 13.into()),
                OpCode::ADD(3, 1, 2),
                OpCode::ADD(4, 0, 3),
                OpCode::MOV(0, 4),
                OpCode::HLT,
            ],
            start_at: 0,
            initial_register: vec![15.into()],
        };
        let x = program.execute(None).expect("valid execution");
        assert_eq!(x, 40.into());
    }
}
